@@ -12,6 +12,18 @@
 #include "vutil.h"
 #include <linux/capability.h>
 
+static int my_chroot(const char *dir)
+{
+	int ret = -1;
+	if (has_chrootsafe()){
+		ret = call_chrootsafe(dir);
+	}else{
+		fprintf (stderr,"Kernel do not support chrootsafe(), using chroot()\n");
+		ret = chroot (dir);
+	}
+	return ret;
+}
+
 int main (int argc, char *argv[])
 {
 	if (argc < 3){
@@ -46,12 +58,12 @@ int main (int argc, char *argv[])
 		// and also a security flaw. The shared objects in the vserver
 		// may be tweaked to get control of the root server ...
 		getpwnam ("root");
-		if (chroot (argv[dir]) == -1){
+		if (my_chroot (argv[dir]) == -1){
 			fprintf (stderr,"Can't chroot to directory %s (%s)\n",argv[dir]
 				,strerror(errno));
 		}else{
 			if (nochroot){
-				call_new_s_context (-2,1<<CAP_SYS_CHROOT,0);
+				call_new_s_context (0,NULL,1<<CAP_SYS_CHROOT,0);
 			}
 
 			struct passwd *p = NULL;
@@ -68,9 +80,18 @@ int main (int argc, char *argv[])
 				setuid(p->pw_uid);
 			}
 			int cmd = dir + 1;
-			execvp (argv[cmd],argv+cmd);
-			fprintf (stderr,"Can't execute %s (%s)\n",argv[cmd]
-				,strerror(errno));
+			if (cmd >= argc){
+				fprintf (stderr,"capchroot: No command to execute, do nothing\n");
+			}else{
+				#if 0
+				for (int i=cmd; i<argc; i++){
+					printf ("argv[%d]=:%s:\n",i,argv[i]);
+				}
+				#endif
+				execvp (argv[cmd],argv+cmd);
+				fprintf (stderr,"Can't execute %s (%s)\n",argv[cmd]
+					,strerror(errno));
+			}
 		}
 	}
 	return -1;
