@@ -18,7 +18,7 @@
 static void usage()
 {
 	fprintf (stderr,"chbind version %s\n",VERSION);
-	fprintf (stderr,"chbind [ --silent ] [ --ip ip_num ] [ --bcast broadcast ] command argument\n");
+	fprintf (stderr,"chbind [ --silent ] [ --ip ip_num[/mask] ] [ --bcast broadcast ] command argument\n");
 	exit (-1);
 }
 
@@ -68,6 +68,7 @@ static int ifconfig_ioctl(
 int ifconfig_getaddr (
 	const char *ifname,
 	unsigned long &addr,
+	unsigned long &mask,
 	unsigned long &bcast)
 {
 	int ret = -1;
@@ -82,6 +83,11 @@ int ifconfig_getaddr (
 				addr = sin->sin_addr.s_addr;
 				ret = 0;
 			}
+			if (ifconfig_ioctl(skfd,ifname,SIOCGIFNETMASK, &ifr) >= 0){
+				struct sockaddr_in *sin = (struct sockaddr_in*)&ifr.ifr_addr;
+				mask = sin->sin_addr.s_addr;
+				ret = 0;
+			}
 			if (ifconfig_ioctl(skfd,ifname,SIOCGIFBRDADDR, &ifr) >= 0){
 				struct sockaddr_in *sin = (struct sockaddr_in*)&ifr.ifr_addr;
 				bcast = sin->sin_addr.s_addr;
@@ -101,33 +107,63 @@ int main (int argc, char *argv[])
 	int ret = -1;
 	int silent = 0;
 	int i;
-	unsigned long addrs[16];
+	unsigned long addrs[16],masks[16];
 	int nbaddrs = 0;
 	unsigned long bcast = 0xffffffff;
 	for (i=1; i<argc; i++){
 		const char *arg = argv[i];
 		const char *opt = argv[i+1];
 		if (strcmp(arg,"--ip")==0){
-			unsigned long addr;
+			unsigned long addr,mask;
 			if (nbaddrs == 16){
 				fprintf (stderr,"Too many IP numbers, max 16, ignored\n");
 
-			}else if (ifconfig_getaddr(opt,addr,bcast)==-1){
+			}else if (ifconfig_getaddr(opt,addr,mask,bcast)==-1){
+				unsigned long mask = 0x00ffffff;
+				const char *pt = strchr(opt,'/');
+				char tmpopt[strlen(opt)+1];
+				if (pt != NULL){
+					strcpy (tmpopt,opt);
+					tmpopt[pt-opt] = '\0';
+					opt = tmpopt;
+					pt++;
+					if (strchr(pt,'.')==NULL){
+						// Ok, we have a network size, not a netmask
+						int size = atoi(pt);
+						mask = 0;
+						for (int i=0; i<size; i++){
+							mask = mask >> 1;
+							mask |= 0x80000000;
+						}
+						mask = ntohl(mask);
+					}else{
+						struct hostent *h = gethostbyname (pt);
+						if (h != NULL){
+							memcpy (&mask,h->h_addr,sizeof(mask));
+						}else{
+							fprintf (stderr,"Invalid netmask: %s\n",pt);
+							usage();
+						}
+					}
+							
+				}
 				struct hostent *h = gethostbyname (opt);
 				if (h == NULL){
 					fprintf (stderr,"Invalid IP number or host name: %s\n",opt);
 					usage();
 				}else{
 					memcpy (&addr,h->h_addr,sizeof(addr));
+					masks[nbaddrs] = mask;
 					addrs[nbaddrs++] = addr;
 				}
 			}else{
+				masks[nbaddrs] = mask;
 				addrs[nbaddrs++] = addr;
 			}
 			i++;
 		}else if (strcmp(arg,"--bcast")==0){
 			unsigned long tmp;
-			if (ifconfig_getaddr(opt,tmp,bcast)==-1){
+			if (ifconfig_getaddr(opt,tmp,tmp,bcast)==-1){
 				struct hostent *h = gethostbyname (opt);
 				if (h == NULL){
 					fprintf (stderr,"Invalid broadcast number: %s\n",opt);
@@ -148,7 +184,7 @@ int main (int argc, char *argv[])
 	}else if (argv[i][0] == '-'){
 		usage();
 	}else{
-		if (call_set_ipv4root(addrs,nbaddrs,bcast)==0){
+		if (call_set_ipv4root(addrs,nbaddrs,bcast,masks)==0){
 			if (!silent){
 				printf ("ipv4root is now");
 				for (int i=0; i<nbaddrs; i++){
