@@ -1,7 +1,8 @@
 #!/bin/sh
 # Fix some path in scripts and then install
 if [ $# != 4 ] ; then
-	echo install-fix.sh file destfile /usr/sbin /usr/lib/vserver
+	echo '*******' install-fix.sh file destfile /usr/sbin /usr/lib/vserver
+	exit 1
 else
 	DEST=$2
 	if [ -d $DEST ] ; then
